package com.ai.bss.es.esmanage.service.impl;

import com.ai.abc.jpa.elastic.model.ElasticPage;
import com.ai.abc.jpa.elastic.model.PageInfo;
import com.ai.abc.jpa.elastic.model.TimeInfo;
import com.ai.abc.jpa.elastic.repository.ElasticEntityRepository;
import com.ai.abc.jpa.elastic.repository.impl.ElasticEntityRepositoryImpl;
import com.ai.bss.es.esmanage.service.EsService;
import com.ai.bss.es.esmanage.utils.RequestEntity;
import com.ai.bss.es.esmanage.utils.ResponseResult;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.TermsQueryBuilder;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.bucket.terms.TermsAggregationBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.IOException;
import java.util.*;


@Slf4j
@Service
public class EsServiceImpl implements EsService {


    @Value("${es.server:Empty}")
    String esServer ;
    @Value("${es.server.username:Empty}")
    String userName ;
    @Value("${es.server.password:Empty}")
    String password ;

//    @Autowired
//    ElasticEntityRepository elasticEntityRepository;

    ElasticEntityRepository elasticEntityRepository = null;

    private ElasticEntityRepository getElasticEntityRepository(){
        if(elasticEntityRepository==null){
//            elasticEntityRepository = new ElasticEntityRepositoryImpl(esServer,userName,password);
            elasticEntityRepository = new ElasticEntityRepositoryImpl();
            log.debug("esServer:"+esServer);
            elasticEntityRepository.initServer(esServer,userName,password);
        }
        return elasticEntityRepository;
    }


    @Override
    public ResponseResult executeEsEntity(RequestEntity requestEntity) throws Exception {

        if(requestEntity==null){
            return null;
        }

        String methods = requestEntity.getMethods();

        if("SAVE".equals(methods)){
            saveEsEntity(requestEntity);
            return ResponseResult.sucess();
        }else if("COUNT".equals(methods)){
            return countSearch(requestEntity,null,methods);
        }else if("COUNTANDGROUP".equals(methods)){
            return countSearch(requestEntity,"GROUP",methods);
        }else if("COUNT_BY_TIME".equals(methods)){
            return countSearch(requestEntity,"GROUP",methods);
        }else{
            return findEsEntityList(requestEntity);
        }

    }

    private void saveEsEntity(RequestEntity requestEntity){

        getElasticEntityRepository().purePersist(requestEntity.getTableName(), "", requestEntity.getEntity());
    }

    private ResponseResult findEsEntityList(RequestEntity requestEntity) {

        log.info("requestEntity:\n"+JSONObject.toJSONString(requestEntity));
        PageInfo pageInfo = new PageInfo();
        int pageSize = requestEntity.getPageSize() == null ? 0 : requestEntity.getPageSize();
        int startNum = requestEntity.getStartNumber() == null ? 0 : requestEntity.getStartNumber()*pageSize;
        pageInfo.setStartNumber(startNum);
        pageInfo.setPageSize(pageSize);
        Map entityMap = null;
        if (!StringUtils.isEmpty(requestEntity.getEntity())) {

            String entityStr = requestEntity.getEntity();
            entityMap = JSONObject.parseObject(entityStr, new TypeReference<Map<String, Object>>() {});
        }


        Map fieldMap = null;
        String entityKey;
        if (entityMap != null && !entityMap.isEmpty()) {
            fieldMap = new HashMap();
            Iterator var10 = entityMap.entrySet().iterator();

            while(var10.hasNext()) {
                Map.Entry<String, Object> entry = (Map.Entry)var10.next();
                entityKey = (String)entry.getKey();
                if (entry.getValue() != null) {
                    TermsQueryBuilder termsQueryBuilder = null;
                    if (entry.getValue().getClass() == JSONArray.class) {
//                        this.log.debug("mapKey=" + sortType + ",entry.getValue()=" + entry.getValue());
//                        termsQueryBuilder = QueryBuilders.termsQuery(sortType, (ArrayList)entry.getValue());
                        fieldMap.put(entityKey,JSONArrayToArrayList((JSONArray)entry.getValue()));
                    } else {
                        String mapValue = entry.getValue().toString();
//                        termsQueryBuilder = QueryBuilders.termsQuery(sortType, new String[]{mapValue});
                        fieldMap.put(entityKey,mapValue);
                    }

                }
//                fieldMap.put(entityKey)
            }
        }


        Map sortMap = new HashMap();
        if(!StringUtils.isEmpty(requestEntity.getSortFieldName())){
            sortMap.put("sortFieldName",requestEntity.getSortFieldName());
            sortMap.put("sortType",requestEntity.getSortType());
        }
        TimeInfo timeInfo =new TimeInfo();
        timeInfo.setStartTime(requestEntity.getStartTime());
        timeInfo.setEndTime(requestEntity.getEndTime());
        timeInfo.setFieldName(requestEntity.getTimeFieldName());

        log.info("es-server: " + esServer);

        ElasticPage elasticPage = getElasticEntityRepository().findEntity(requestEntity.getTableName(), fieldMap, timeInfo, pageInfo,sortMap);

        return ResponseResult.sucess(elasticPage.getTotalSize(),elasticPage.getResults());
    }

    private ArrayList<String> JSONArrayToArrayList(JSONArray jsonArray){
        ArrayList<String> listdata = null;
        if (jsonArray != null) {
            listdata = new ArrayList<String>();
            for (int i=0;i<jsonArray.size();i++){
                listdata.add(jsonArray.getString(i));
            }
        }
        return listdata;
    }

    private Object findEsEntity(){
        return null;
    }


    @Override
    public ResponseResult countSearch(RequestEntity requestEntity,String group,String methods) throws Exception {
        Map entityMap = null;
        if (!StringUtils.isEmpty(requestEntity.getEntity())) {

            String entityStr = requestEntity.getEntity();
            entityMap = JSONObject.parseObject(entityStr, new TypeReference<Map<String, Object>>() {});
        }



        Map fieldMap = null;
        String entityKey;
        if (entityMap != null && !entityMap.isEmpty()) {
            fieldMap = new HashMap();
            Iterator var10 = entityMap.entrySet().iterator();

            while(var10.hasNext()) {
                Map.Entry<String, Object> entry = (Map.Entry)var10.next();
                entityKey = (String)entry.getKey();
                if (entry.getValue() != null) {
                    TermsQueryBuilder termsQueryBuilder = null;
                    if (entry.getValue().getClass() == JSONArray.class) {
                        fieldMap.put(entityKey,JSONArrayToArrayList((JSONArray)entry.getValue()));
                    } else {
                        String mapValue = entry.getValue().toString();
                        fieldMap.put(entityKey,mapValue);
                    }

                }
//                fieldMap.put(entityKey)
            }
        }
        TimeInfo timeInfo =new TimeInfo();
        timeInfo.setStartTime(requestEntity.getStartTime());
        timeInfo.setEndTime(requestEntity.getEndTime());
        timeInfo.setFieldName(requestEntity.getTimeFieldName());

//        if(!StringUtils.isEmpty(group)){
//            List<Map> groupMap = getElasticEntityRepository().countAndSearch(requestEntity.getTableName(),fieldMap,requestEntity.getGroupField(),timeInfo,null);
//            return ResponseResult.sucess(groupMap);
//        }else{
//            Long count = getElasticEntityRepository().countSearch(requestEntity.getTableName(),fieldMap,timeInfo,null);
//            return ResponseResult.sucess(count);
//        }else{
//            Map countMap = getElasticEntityRepository().countAndGroupWhitEventTime(requestEntity.getTableName(),fieldMap,requestEntity.getGroupField(),timeInfo,null);
//            return ResponseResult.sucess(countMap);
//        }

        if("COUNT".equals(methods)){

            Long count = getElasticEntityRepository().countSearch(requestEntity.getTableName(),fieldMap,timeInfo,null);
            return ResponseResult.sucess(count);
        }else if("COUNTANDGROUP".equals(methods)){
            List<Map> groupMap = getElasticEntityRepository().countAndSearch(requestEntity.getTableName(),fieldMap,requestEntity.getGroupField(),timeInfo,null);
            return ResponseResult.sucess(groupMap);
        }else if("COUNT_BY_TIME".equals(methods)){
            Map countMap = getElasticEntityRepository().countAndGroupWhitEventTime(requestEntity.getTableName(),fieldMap,requestEntity.getGroupField(),timeInfo,null);
            return ResponseResult.sucess(countMap);
        }
        return ResponseResult.sucess();
    }

}
