package com.ai.bss.es.esmanage.utils;


import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

/**
 * 接口响应参数公共类
 *
 * @author chencai
 */
@NoArgsConstructor
@Getter
@Setter
public class ResponseResult<T> {
    /**
     * 结果编码
     */
    private String resultCode;

    /**
     * 结果信息
     */
    private String resultMsg;

    /**
     * 调用轨迹标识
     */
    private Long traceId;

    /**
     * 结果集数量
     */
    private Long totalResultNumber;

    /**
     * 结果集信息
     */
    private T result;

    private ResponseResult(String resultCode, String resultMsg) {
        this.resultCode = resultCode;
        this.resultMsg = resultMsg;
    }

    private ResponseResult(String resultCode, String resultMsg, T result) {
        this.resultCode = resultCode;
        this.resultMsg = resultMsg;
        this.result = result;
    }

    private ResponseResult(String resultCode, String resultMsg, Long totalResuleNumber,
                           T result) {
        this.resultCode = resultCode;
        this.resultMsg = resultMsg;
        this.totalResultNumber = totalResuleNumber;
        this.result = result;
    }

    private ResponseResult(String resultCode, String resultMsg, Long traceId, Long totalResuleNumber,
                           T result) {
        this.resultCode = resultCode;
        this.resultMsg = resultMsg;
        this.traceId = traceId;
        this.totalResultNumber = totalResuleNumber;
        this.result = result;
    }
    
    public static <T> ResponseResult<T> sucess() {
        return new ResponseResult<T>(CommonConsts.RESULT_CODE_SUCCESSFUL, CommonConsts.RESULT_MSG_SUCCESSFUL);
    }

    public static <T> ResponseResult<T> sucess(String resultMsg) {
        return new ResponseResult<T>(CommonConsts.RESULT_CODE_SUCCESSFUL, resultMsg);
    }

    public static <T> ResponseResult<T> sucess(T result) {
        return new ResponseResult<T>(CommonConsts.RESULT_CODE_SUCCESSFUL, CommonConsts.RESULT_MSG_SUCCESSFUL, result);
    }

    public static <T> ResponseResult<T> sucess(long totalResuleNumber, T result) {
        return new ResponseResult<T>(CommonConsts.RESULT_CODE_SUCCESSFUL, CommonConsts.RESULT_MSG_SUCCESSFUL, totalResuleNumber, result);
    }

    public static <T> ResponseResult<T> sucess(Long traceId, Long totalResuleNumber, T result) {
        return new ResponseResult<T>(CommonConsts.RESULT_CODE_SUCCESSFUL, CommonConsts.RESULT_MSG_SUCCESSFUL, traceId, totalResuleNumber, result);
    }

    public static <T> ResponseResult<T> error() {
        return new ResponseResult<T>(CommonConsts.RESULT_CODE_FAILED, CommonConsts.RESULT_MSG_FAILED);
    }

    public static <T> ResponseResult<T> error(String errorMsg) {
        return new ResponseResult<T>(CommonConsts.RESULT_CODE_FAILED, errorMsg);
    }

    public static <T> ResponseResult<T> error(String resultCode, String errorMsg) {
        return new ResponseResult<T>(resultCode, errorMsg);
    }

    public static <T> ResponseResult<T> error(String resultCode, String errorMsg, Long traceId) {
        return new ResponseResult<T>(resultCode, errorMsg, traceId, 0L, null);
    }
}
